using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;

namespace Microsoft.SharedSource.SqlServer.MDDE.Components
{
    [
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1705:LongAcronymsShouldBePascalCased"),
        SupportedComponentType("Microsoft.SqlServer.Dts.Tasks.ExecuteSQLTask.ExecuteSQLTask, Microsoft.SqlServer.SQLTask, Version=9.0.242.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91")
    ]
    public class MDDETask: MDDEContainer
    {
        #region Member Variables

        private XmlNodeList changedProperties = null;
        private TaskHost component = null;

        #endregion

        public MDDETask(Package package, object component, string name, string uri)
            : base(package, component, name, uri)
        {
            if (!(component is TaskHost))
                throw new ArgumentException();
        }

        #region Overridden Methods

        public override void GetChangeXml(MDDEComponent subject, XmlDocument changeXml)
        {
            #region Set Values

            if (subject == null)
                throw new ArgumentNullException("Subject can not NULL");

            if (!(subject is MDDETask))
                throw new ArgumentException("Subject is not of type MDDETask");

            component = (TaskHost)innerComponent;
            TaskHost referenceComponent = (TaskHost)subject.InnerComponent;

            DtsProperties componentPropertyCollection = (DtsProperties)component.Properties;
            DtsProperties referenceComponentPropertyCollection = (DtsProperties)referenceComponent.Properties;

            XmlNode componentNode = null;
            componentNode = changeXml.CreateElement("Component");

            bool nodeEmpty = true;

            #endregion

            #region Compare properties of Component with subject and create Properties' Xml

            for (int propertyIndex = 0; propertyIndex < componentPropertyCollection.Count; propertyIndex++)
            {
                if (componentPropertyCollection[propertyIndex].Name.CompareTo("Name") == 0)
                    continue;

                DtsProperty componentProperty = componentPropertyCollection[propertyIndex];
                
                if (!referenceComponentPropertyCollection.Contains(componentProperty.Name))
                    continue;
                
                DtsProperty referenceComponentProperty = referenceComponentPropertyCollection[componentProperty.Name];

                if ((componentProperty.Get && componentProperty.Set)
                    && (referenceComponentProperty.Get && referenceComponentProperty.Set))
                {
                    string componentPropertyValue = String.Empty;
                    if (componentProperty.GetValue(component) != null)
                        componentPropertyValue = componentProperty.GetValue(component).ToString();

                    string referenceComponentPropertyValue = String.Empty;
                    if (referenceComponentProperty.GetValue(referenceComponent) != null)
                        referenceComponentPropertyValue = referenceComponentProperty.GetValue(referenceComponent).ToString();

                    if (componentPropertyValue != referenceComponentPropertyValue)
                    {
                        UpdateNodeProperties(changeXml, componentNode, componentProperty.Name, componentPropertyValue);
                        nodeEmpty = false;
                    }
                }
            }

            #endregion

            #region Update Changes Xml

            if (!nodeEmpty)
            {
                AddAttribute(changeXml, componentNode, "Name", this.name);
                AddAttribute(changeXml, componentNode, "URL", this.uri);
                AddAttribute(changeXml, componentNode, "RowsetDescriptor", "");

                changeXml.FirstChild.AppendChild(componentNode);
            }

            #endregion

            #region Trace Successors and Update Changes Xml

            this.innerExecutionDAG.GetChangeXml(((MDDETask)subject).innerExecutionDAG, changeXml);

            #endregion
        }

        public override bool ApplyChanges(XmlDocument changeXml, MDDEExecutionDAG originalPackage)
        {
            try
            {
                #region Apply Changes to Inner Execution DAG

                this.innerExecutionDAG.ApplyChanges(changeXml, originalPackage);

                #endregion

                #region Set Values

                XmlNode componentChangeXml =
                    changeXml.SelectSingleNode("//PackageChanges/Component[@URL='" + this.uri + "']");

                //if (innerComponent == null)
                //    return false;

                //if (!(innerComponent is TaskHost))
                //    return false;

                component = (TaskHost)innerComponent;

                DtsProperties dtsProperties = (DtsProperties)component.Properties;

                #endregion

                #region Read Change Xml

                if (componentChangeXml != null)
                {
                    if (componentChangeXml.SelectSingleNode("Properties") != null)
                        changedProperties = componentChangeXml["Properties"].ChildNodes;
                }

                #endregion

                #region Apply Properties and Verify If Sufficient Property Values' Are Available

                if (componentChangeXml != null)
                {
                    ApplyProperties(dtsProperties);
                }

                #endregion
                                
                return true;
            }

            catch
            {
                // TODO: Add to Log
                return false;
            }
        }

        public override bool Validate()
        {
            return base.Validate();
        }

        #endregion

        #region Helper Methods

        protected void ApplyProperties(DtsProperties dtsProperties)
        {
            if (changedProperties != null)
            {
                foreach (XmlNode changedProperty in changedProperties)
                {
                    dtsProperties[changedProperty.Attributes["Name"].Value].SetValue(component,
                        ConvertToCompatibleType(changedProperty.Attributes["Value"].Value, dtsProperties[changedProperty.Attributes["Name"].Value].Type.ToString()));
                }
            }            
        }                

        #endregion
    }
}
