using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using System.Data;
using Microsoft.ApplicationBlocks.Data;
using System.Xml;

namespace Microsoft.SharedSource.SqlServer.MDDE.Repository
{
    public sealed class InstanceColumnCollectionProperty
    {
        #region Fields

        InstanceColumnCollection parentCollection;
        string name;
        string value;

        #endregion

        #region Properties

        public InstanceColumnCollection ParentCollection
        {
            get { return parentCollection; }
        }

        public string Name
        {
            get { return name; }
        }

        public string Value
        {
            get { return this.value; }
            set { this.value = value; }
        }

        #endregion

        #region Methods
        public InstanceColumnCollectionProperty(
            InstanceColumnCollection parentCollection, string propertyName)
            : this(parentCollection, propertyName, null) { }

        public InstanceColumnCollectionProperty(
            InstanceColumnCollection parentCollection,
            string propertyName, SqlTransaction txn)
        {
            this.parentCollection = parentCollection;
            this.name = propertyName;

            // See if we can get this collection property from repository.
            SqlParameter[] sqlParams = new SqlParameter[2];
            sqlParams[0] =
                new SqlParameter("collectionId",
                parentCollection.Id);
            sqlParams[1] = new SqlParameter("propertyName",
                this.Name);

            DataSet propertyInfo;
            if (txn == null)
                propertyInfo =
                SqlHelper.ExecuteDataset(
                parentCollection.ParentComponent.ParentInstance.
                ParentTemplate.ParentRepository.RepositoryConnection,
                CommandType.Text,
                Resource.GetCollectionPropertyInfo, sqlParams);
            else
                propertyInfo =
                SqlHelper.ExecuteDataset(txn,
                CommandType.Text,
                Resource.GetCollectionPropertyInfo, sqlParams);

            // Return if we find nothing.
            if (propertyInfo.Tables[0].Rows.Count == 0)
                return;

            // Otherwise, set the property info.
            this.value =
                propertyInfo.Tables[0].Rows[0]["PropertyValue"].ToString();
        }

        internal void Create(SqlTransaction inputTxn)
        {
            // Create a new transaction if there is no input transaction.
            SqlTransaction txn = inputTxn != null ? inputTxn :
                parentCollection.ParentComponent.
                ParentInstance.ParentTemplate.ParentRepository.
                RepositoryConnection.BeginTransaction();

            try
            {
                SqlParameter[] sqlParams = new SqlParameter[3];
                sqlParams[0] =
                    new SqlParameter("collectionId",
                    parentCollection.Id);
                sqlParams[1] =
                    new SqlParameter("propertyName",
                    this.Name);
                sqlParams[2] =
                    new SqlParameter("propertyValue",
                    this.Value);

                SqlHelper.ExecuteNonQuery(txn,
                    CommandType.StoredProcedure,
                    "sp_CreateColumnCollectionProperty",
                    sqlParams);

                // Commit the transaction if it is not an input one.
                if (inputTxn == null)
                    txn.Commit();
            }
            catch (Exception ex)
            {
                // Rollback the transaction if it is not an input one.
                if (inputTxn == null)
                    txn.Rollback();
                throw ex;
            }
        }

        internal void Update(SqlTransaction txn)
        {
            SqlParameter[] sqlParams = new SqlParameter[3];
            sqlParams[0] =
                new SqlParameter("collectionId",
                parentCollection.Id);
            sqlParams[1] =
                new SqlParameter("propertyName",
                this.Name);
            sqlParams[2] =
                new SqlParameter("propertyValue",
                this.Value);

            SqlHelper.ExecuteNonQuery(txn,
                CommandType.StoredProcedure,
                "sp_UpdateColumnCollectionProperty",
                sqlParams);
        }

        internal void Delete(SqlTransaction txn)
        {
            // Delete property.
            SqlParameter[] sqlParams = new SqlParameter[2];
            sqlParams[0] = new SqlParameter(
                "collectionId", parentCollection.Id);
            sqlParams[1] = new SqlParameter(
                "propertyName", this.Name);

            SqlHelper.ExecuteNonQuery(txn,
                CommandType.StoredProcedure,
                "sp_DeleteColumnCollectionProperty",
                sqlParams);
        }

        #endregion

        internal void ApplyProperties(XmlNode xProperty)
        {
            this.value = xProperty.Attributes["Value"].Value;
        }
    }

    public sealed class InstanceColumnCollectionProperties : IEnumerable
    {
        #region Fields

        private Dictionary<string, InstanceColumnCollectionProperty>
            collectionProperties =
            new Dictionary<string, InstanceColumnCollectionProperty>();
        InstanceColumnCollection parentCollection;

        #endregion

        #region Properties

        public int Count
        {
            get { return collectionProperties.Count; }
        }

        public InstanceColumnCollection ParentCollection
        {
            get { return parentCollection; }
        }

        public InstanceColumnCollectionProperty this[string propertyName]
        {
            get { return collectionProperties[propertyName]; }
        }

        #endregion

        #region Methods

        internal InstanceColumnCollectionProperties(
            InstanceColumnCollection parentCollection)
        {
            this.parentCollection = parentCollection;
        }

        public void Add(InstanceColumnCollectionProperty collectionProperty)
        {
            collectionProperties.Add(
                collectionProperty.Name, collectionProperty);
        }

        public void Remove(InstanceColumnCollectionProperty collectionProperty)
        {
            collectionProperties.Remove(collectionProperty.Name);
        }

        public void Clear()
        {
            collectionProperties.Clear();
        }

        public bool ContainsProperty(string propertyName)
        {
            return collectionProperties.ContainsKey(propertyName);
        }

        #endregion

        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return collectionProperties.Values.GetEnumerator();
        }

        #endregion

    }
}
